package com.yunzhi.niucai.controller;

import com.alipay.api.response.AlipayTradeAppPayResponse;
import com.github.binarywang.wxpay.bean.notify.WxPayOrderNotifyResult;
import com.github.binarywang.wxpay.bean.result.WxPayUnifiedOrderResult;
import com.github.binarywang.wxpay.exception.WxPayException;
import com.yunzhi.niucai.alipay.AlipayFactory;
import com.yunzhi.niucai.alipay.AliPayService;
import com.yunzhi.niucai.alipay.vo.AliPayParam;
import com.yunzhi.niucai.common.*;
import com.yunzhi.niucai.entity.TaOrder;
import com.yunzhi.niucai.entity.TaOrderPay;
import com.yunzhi.niucai.service.ITaOrderPayService;
import com.yunzhi.niucai.service.ITaOrderService;
import com.yunzhi.niucai.service.ITaShopAppService;
import com.yunzhi.niucai.wxpay.WxPayService;
import com.yunzhi.niucai.wxpay.WxpayFactory;
import com.yunzhi.niucai.wxpay.vo.WxPayParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;

@Slf4j
@Api(tags = "支付相关接口")
@RestController
@RequestMapping("/")
public class PayController extends BaseController {

    @Autowired
    public ITaShopAppService iTaShopAppService;

    @Autowired
    private AlipayFactory alipayFactory;

    @Autowired
    private WxpayFactory wxpayFactory;

    @Autowired
    private ITaOrderService iTaOrderService;

    @Autowired
    private ITaOrderPayService iTaOrderPayService;

    /**
     * 手机app获取唤醒支付宝的参数
     * @param request
     * @param taOrder
     * @return
     * @throws Exception
     */
    @PostMapping("/app/order/{payClient}/buylottery")
    @ApiOperation(value = "创建订单", notes = "创建订单", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean createOrder(@ApiParam(value = "支付类型", allowableValues = "wxpay,alipay") @PathVariable String payClient,
                                    @ApiParam("下单记录") @RequestBody TaOrder taOrder,
                                    HttpServletRequest request) throws Exception {

        // 简单校验
        if (null == taOrder || null == taOrder.getOrderId()) {
            return ResponseBean.error("下单记录不存在", ResponseBean.ERROR_MISSING_PARAMS);
        }

        if (StringUtils.isEmpty(payClient)) {
            return ResponseBean.error("请确认支付客户端类型", ResponseBean.ERROR_MISSING_PARAMS);
        }

        if (!"wxpay".equals(payClient) && !"alipay".equals(payClient)) {
            return ResponseBean.error("当前仅支持微信及支付宝", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        // c2b 代表客户支付给商家
        TaOrderPay taOrderPay = iTaOrderPayService.createdBy(taOrder, payClient, "c2b");
        if (null == taOrderPay) {
            return ResponseBean.error("准备交易记录失败", ResponseBean.ERROR_UNAVAILABLE);
        }

        // 支付宝支付
        if ("alipay".equals(payClient)) {
            String totalAmount = String.format("%.2f", (float) taOrder.getCharge() / 100);
            AliPayParam alipayParam = new AliPayParam()
                    .setSubject("购彩")
                    .setOutTradeNo(taOrderPay.getTradeNo())
                    .setTotalAmount(totalAmount);

            AliPayService aliPayService = alipayFactory.getService(taOrder.getShopId());
            AlipayTradeAppPayResponse response = aliPayService.Pay(alipayParam);
            if (null == response) {
                return ResponseBean.error("生成支付宝交易失败", ResponseBean.ERROR_UNAVAILABLE);
            }

//            taOrderPay.setH5CallbackUrl(aliPayService.getCallbackURL(response));
        } else {
            // 微信支付
            WxPayParam wxPayParam = new WxPayParam()
                    .setBody("购彩")
                    .setOutTradeNo(taOrderPay.getTradeNo())
                    .setTotalFee(taOrder.getCharge())
                    .setTimeStart(DateUtils.dateTimeCompact(LocalDateTime.now()))
                    .setTimeExpire(DateUtils.dateTimeCompact(LocalDateTime.now().plusMinutes(30))); // 有效期半小时

            WxPayService wxPayService = wxpayFactory.getService(taOrder.getShopId());
            WxPayUnifiedOrderResult response = wxPayService.pay(request, wxPayParam);
            taOrderPay.setH5CallbackUrl(response.getMwebUrl());
        }

        taOrder.setPayStatus(CommConstants.SRTATUS_PAYING);
        taOrderPay.setStatus(CommConstants.SRTATUS_PAYING);

        if (!iTaOrderService.updateById(taOrder) || !iTaOrderPayService.updateById(taOrderPay)) {
            return ResponseBean.error("更新订单支付信息失败", ResponseBean.ERROR_UNAVAILABLE);
        }

        return ResponseBean.success(taOrderPay);
    }

    /**
     * 支付成功回调
     * @param request
     */
    @GetMapping("/app/callback/{payClient}/pay/{outTradeNo}")
    public void alipayCallback(@PathVariable String payClient,
                               @PathVariable String outTradeNo,
                               @RequestBody String reqBody,
                               HttpServletRequest request){

        TaOrder taOrder = iTaOrderService.getByOutTradeNo(outTradeNo);
        if (null == taOrder) {
            log.error("解析支付回调失败: 订单号 [{}]", outTradeNo);
        }

        String payStatus;
        if ("alipay".equals(payClient)) {
            payStatus = request.getParameter("trade_status");
        } else {
            try {
                WxPayService wxPayService = wxpayFactory.getService(taOrder.getShopId());
                WxPayOrderNotifyResult orderNotifyResult = wxPayService.getOrderNotifyResult(reqBody);
                payStatus = orderNotifyResult.getResultCode();
            } catch (WxPayException e) {
                log.error("解析回调支付结果失败: 订单号 [{}]", outTradeNo);
                e.printStackTrace();
            }
        }
    }

    private void alipayCallBack(HttpServletRequest request) {

//        String tradeNo = request.getParameter("trade_no");
//        String appId = request.getParameter("app_id");
//        String outTradeNo = request.getParameter("out_trade_no");
//        String tradeStatus = request.getParameter("trade_status");
//        aliPayService.payCallback(tradeNo,outTradeNo,tradeStatus);

    }
}
